////////////////////////////////////////////////////////////////////////////
//
// mNbN.
// pdNbN擾.
//
// Copyright by rubidesu 2017
// http://www.gatelink.co.jp/hw/
//
////////////////////////////////////////////////////////////////////////////

#include <16f87.h>
#include "eeprom_addr.h"

#define _SERIAL_DEBUG_
//#undef _SERIAL_DEBUG_

//#define _AUDIOO_FREQ_TEST_
#undef _AUDIOO_FREQ_TEST_

#fuses HS, NOLVP, NOWDT, PUT, NOPROTECT, BROWNOUT, MCLR, CCPB3
#use delay(clock=20000000)
#use i2c(MASTER, SDA=PIN_B6, SCL=PIN_B7)
#use RS232(BAUD=9600, XMIT=PIN_B5, RCV=PIN_B2)

typedef unsigned int1	BIT;
typedef unsigned int16	WORD;
typedef unsigned int32	DWORD;

#define LOW		0
#define HIGH	1

struct tagPORTA{
	short bit0;
	short bit1;
	short bit2;
	short bit3;
	short bit4;
	short bit5;
	short bit6;
	short bit7;
}PORTA;
#byte PORTA = 5

struct tagPORTB{
	short bit0;
	short bit1;
	short bit2;
	short bit3;
	short bit4;
	short bit5;
	short bit6;
	short bit7;
}PORTB;
#byte PORTB = 6

//Mf[^obt@.
#define RCV_MAX		64			//Mobt@TCY.
BYTE rcv_stack[RCV_MAX];		//Mobt@.
BYTE rcv_read = 0;				//Mobt@̃[h|C^.
BYTE rcv_write = 0;				//Mobt@̃Cg|C^.
BYTE rcv_buff;					//Mobt@󂯓npϐ.

//|[g`.
#define AMP_POWER	PORTA.bit3
#define POWER_ON	LOW
#define POWER_OFF	HIGH

#define PUSH_SW1	PORTA.bit0
#define PUSH_SW2	PORTA.bit1
#define PUSH_SW3	PORTA.bit2

//EEPROM̃AhX.
#define EEPROM_DEVICE_ADDR	0xa0

//I2C̒萔.
#define IIC_WRITE	0x00
#define IIC_READ	0x01

//WX^AhX.
#byte TMR0 = 0x01
#byte CCP1CON = 0x17
#byte CCPR1L = 0x15
#bit TMR0IF = 0x0b.2

//vJE^p̒萔ƕϐ.
#define COUNT_PER_HOUR		900		//1ԓ̃JEg.
#define COUNT_PER_MINUTE	15		//1̃JEg.
#define COUNT_SEC			4		//1JEg̕b.
#define MAX_SECOND			10800	//15 * 60 * 12.
WORD wSecondCount = 0;				//vJE^.
BYTE bMaxPulse = 0;					//^C}0ɐݒ肷l.

//ݎ̎擾p.
BYTE bHour;
BYTE bMinute;
BYTE bSecond;

#define BEEP_MAX	30
const BYTE beep_tbl[BEEP_MAX] = {	//r[vpgǂ.
	80,	//00.
	89,	//01.
	96,	//02.
	101	//03.,
	105	//04.,
	107	//05.,
	109	//06.,
	110	//07.,
	110	//08.,
	109	//09.,
	107	//10.,
	105	//11.,
	101	//12.,
	96,	//13.
	89,	//14.
	80,	//15.
	71,	//16.
	64,	//17.
	59,	//18.
	55,	//29.
	53,	//20.
	51,	//21.
	50,	//22.
	50,	//23.
	51,	//24.
	53,	//25.
	55,	//26.
	59,	//27.
	64,	//28.
	71,	//29.
};

////////////////////////////////////////////////////////////////////////////
// v荞.
////////////////////////////////////////////////////////////////////////////
#INT_RTCC
void int_rtcc_isr(void)
{
	TMR0 = bMaxPulse;

	wSecondCount += 1;
	wSecondCount %= MAX_SECOND;
}

////////////////////////////////////////////////////////////////////////////
// VAM荞.
////////////////////////////////////////////////////////////////////////////
#INT_RDA
void int_rda_isr(void)
{
	rcv_stack[rcv_write] = getc();

	rcv_write++;
	rcv_write = rcv_write % RCV_MAX;
}

////////////////////////////////////////////////////////////////////////////
// VAMf[^̎擾.
////////////////////////////////////////////////////////////////////////////
BYTE getc_buff(void)
{
	if(rcv_read != rcv_write){
		rcv_buff = rcv_stack[rcv_read];
		rcv_read++;
		rcv_read = rcv_read % RCV_MAX;
		return 1;
	}
	else{
		return 0;
	}
}

////////////////////////////////////////////////////////////////////////////
// 擾.
////////////////////////////////////////////////////////////////////////////
void get_time(void)
{
	WORD wTotalSecond;

	wTotalSecond = wSecondCount;

	bHour = (BYTE)(wTotalSecond / COUNT_PER_HOUR);
	wTotalSecond = wTotalSecond % COUNT_PER_HOUR;

	bMinute = (BYTE)(wTotalSecond / COUNT_PER_MINUTE);
	wTotalSecond = wTotalSecond % COUNT_PER_MINUTE;

	bSecond = (BYTE)wTotalSecond;
}

////////////////////////////////////////////////////////////////////////////
// ݒ.
////////////////////////////////////////////////////////////////////////////
void set_time(BYTE bHour, BYTE bMinute, BYTE bSecond)
{
	WORD wTotalSecond;

	wTotalSecond = (WORD)bHour * COUNT_PER_HOUR;

	wTotalSecond += (WORD)bMinute * COUNT_PER_MINUTE;

	wTotalSecond += (WORD)bSecond;

	wSecondCount = wTotalSecond;
}

////////////////////////////////////////////////////////////////////////////
// .
////////////////////////////////////////////////////////////////////////////
BIT get_sw1()
{
	BIT sw_state = LOW;

	if(PUSH_SW1 == LOW){		//DOWN
		delay_ms(10);
		if(PUSH_SW1 == LOW){
			for(;;){
				if(PUSH_SW1 == HIGH){
					break;
				}
			}
			sw_state = HIGH;
		}
	}

	return sw_state;
}

////////////////////////////////////////////////////////////////////////////
// .
////////////////////////////////////////////////////////////////////////////
BIT get_sw2()
{
	BIT sw_state = LOW;

	if(PUSH_SW2 == LOW){		//DOWN
		delay_ms(10);
		if(PUSH_SW2 == LOW){
			for(;;){
				if(PUSH_SW2 == HIGH){
					break;
				}
			}
			sw_state = HIGH;
		}
	}

	return sw_state;
}

////////////////////////////////////////////////////////////////////////////
// }jA.
////////////////////////////////////////////////////////////////////////////
BIT get_sw3()
{
	BIT sw_state = LOW;

	if(PUSH_SW3 == LOW){		//DOWN
		delay_ms(10);
		if(PUSH_SW3 == LOW){
			for(;;){
				if(PUSH_SW3 == HIGH){
					break;
				}
			}
			sw_state = HIGH;
		}
	}

	return sw_state;
}

////////////////////////////////////////////////////////////////////////////
// ̓ǂݏグ.
////////////////////////////////////////////////////////////////////////////
void PlaySound(BYTE bAddrIndex)
{
	BYTE bReadData;
	DWORD dwAddress;

	if(bAddrIndex == 255){
		return;
	}

	dwAddress = voice_addr[bAddrIndex];

	i2c_start();
	if(dwAddress > 0xffff){
		i2c_write(EEPROM_DEVICE_ADDR | 0x02 | IIC_WRITE);
	}
	else{
		i2c_write(EEPROM_DEVICE_ADDR | IIC_WRITE);
	}

	i2c_write((BYTE)((dwAddress >> 8) & 0xff));
	i2c_write((BYTE)(dwAddress & 0xff));

	i2c_start();
	if(dwAddress > 0xffff){
		i2c_write(EEPROM_DEVICE_ADDR | 0x02 | IIC_READ);
	}
	else{
		i2c_write(EEPROM_DEVICE_ADDR | IIC_READ);
	}

#ifndef _AUDIOO_FREQ_TEST_
	for(;(bReadData = i2c_read()) != 0xff ;){
#else
	for(;(bReadData = i2c_read()) != 0x80 ;){
#endif //_AUDIOO_FREQ_TEST_
		CCPR1L = bReadData;
		delay_us(25);
	}

	i2c_stop();
}

////////////////////////////////////////////////////////////////////////////
// r[v.
////////////////////////////////////////////////////////////////////////////
void PlayBeep(BYTE bNum)
{
	BYTE n;
	BYTE i;

	for(; bNum > 0 ; bNum--){
		for(n = 0; n < 100; n++){
			for(i = 0; i < BEEP_MAX; i++){
				CCPR1L = beep_tbl[i];
				delay_us(15);
			}
		}
		delay_ms(112);
	}
}

////////////////////////////////////////////////////////////////////////////
// Av̓dON/OFF.
////////////////////////////////////////////////////////////////////////////
void AmpPowerControl(BYTE bOnOff)
{
	AMP_POWER = bOnOff;

	if(bOnOff == POWER_OFF){
		CCPR1L = 0x80;
	}
}

////////////////////////////////////////////////////////////////////////////
// C.
//
// IN:
//
// OUT:
//
// RETURN:
//
////////////////////////////////////////////////////////////////////////////
main()
{
	BYTE bLastSecond = 0;
	BYTE bLastMinute = 0;
	BYTE i;

	WORD wSyncCount = 0;
	BYTE bSyncIndex = 0;

	set_tris_a(0b10110111);
	set_tris_b(0b00000100);

	setup_spi(SPI_SS_DISABLED); 
	setup_comparator(NC_NC_NC_NC); 

	CCP1CON = CCP_PWM;
	setup_timer_2(T2_DIV_BY_1, 0xff, 1);
	CCPR1L = (BYTE)0x00;

	AmpPowerControl(POWER_OFF);

	delay_ms(100);

#ifdef _AUDIOO_FREQ_TEST_
	AmpPowerControl(POWER_ON);
	for(;;){
		PlaySound(0);
	}
#endif //_AUDIOO_FREQ_TEST_

	disable_interrupts(INT_RTCC);
	disable_interrupts(GLOBAL);

	//̊荞ݗ} kuniku no saku.
	TMR0IF = 0;

	//ȈՎgJE^.
	setup_timer_0(RTCC_EXT_L_TO_H | RTCC_DIV_1);
	set_timer0(0);
	delay_ms(1000);			//ACǧv.
	wSecondCount = get_timer0();

	if((wSecondCount >= 48) && (wSecondCount <= 52)){
		bMaxPulse = 50;	//50Hz.

		//50HzF(r[v1).
		AmpPowerControl(POWER_ON);
		PlayBeep(1);
		AmpPowerControl(POWER_OFF);
	}
	else if((wSecondCount >= 58) && (wSecondCount <= 62)){
		bMaxPulse = 60;	//60Hz.

		//60HzF(r[v2).
		AmpPowerControl(POWER_ON);
		PlayBeep(2);
		AmpPowerControl(POWER_OFF);
	}
	#ifdef _SERIAL_DEBUG_
	printf("%02UHz(%03Upulse)\r\n", bMaxPulse, (BYTE)wSecondCount);
	#endif //_SERIAL_DEBUG_

	if(bMaxPulse == 0){
		//ُ~(r[v5).
		AmpPowerControl(POWER_ON);
		PlayBeep(5);
		AmpPowerControl(POWER_OFF);

		while(1){
			;
		}
	}

	bMaxPulse = (BYTE)((WORD)256 - ((WORD)bMaxPulse * (WORD)COUNT_SEC));

	wSecondCount = 0;
	setup_timer_0(RTCC_EXT_L_TO_H | RTCC_DIV_1);
	set_timer0(bMaxPulse);

	enable_interrupts(INT_RDA);
	enable_interrupts(INT_RTCC);
	enable_interrupts(GLOBAL);

	if(get_sw3() == TRUE){
		AmpPowerControl(POWER_ON);
		for(i = 0; i < MAX_PHRASE; i++){
			PlaySound(i);
		}
		PlayBeep(1);
		AmpPowerControl(POWER_OFF);
	}

	while(1){
		///////////////////////////////////////////////////////
		// XCb` ///////////////////////////////////////
		if(get_sw1() == TRUE){	//.
			get_time();

			bHour++;
			bHour %= 12;

			set_time(bHour, bMinute, bSecond);

			#ifdef _SERIAL_DEBUG_
			printf("%02U:%02U:%02U\r\n", bHour, bMinute, bSecond * COUNT_SEC);
			#endif //_SERIAL_DEBUG_

			AmpPowerControl(POWER_ON);
			PlayBeep(1);
			PlaySound(hour_tbl[bHour]);
			AmpPowerControl(POWER_OFF);
		}
		if(get_sw2() == TRUE){	//.
			get_time();

			bMinute++;
			bMinute %= 60;

			bSecond = 0;
			set_timer0(bMaxPulse);

			set_time(bHour, bMinute, bSecond);

			#ifdef _SERIAL_DEBUG_
			printf("%02U:%02U:%02U\r\n", bHour, bMinute, bSecond * COUNT_SEC);
			#endif //_SERIAL_DEBUG_

			bLastMinute = bMinute;	//̎}~.
			bLastSecond = bSecond;

			AmpPowerControl(POWER_ON);
			PlayBeep(1);
			if(bMinute == 0){
				PlaySound(hour_tbl[bHour]);
			}
			PlaySound(minute_tbl[bMinute][0]);
			PlaySound(minute_tbl[bMinute][1]);
			AmpPowerControl(POWER_OFF);
		}
		if(get_sw3() == TRUE){	//}jA.
			get_time();

			AmpPowerControl(POWER_ON);
			PlayBeep(1);
			delay_ms(500);
			PlaySound(hour_tbl[bHour]);
			PlaySound(minute_tbl[bMinute][0]);
			PlaySound(minute_tbl[bMinute][1]);
			PlaySound(LAST_VOICE);
			AmpPowerControl(POWER_OFF);
		}

		///////////////////////////////////////////////////////
		// PC̎ ///////////////////////////////////
		if(getc_buff() != 0){
			switch(rcv_buff){
			case '0':
			case '1':
			case '2':
			case '3':
			case '4':
			case '5':
			case '6':
			case '7':
			case '8':
			case '9':
				{	//JE^lM.
					switch(bSyncIndex){
					case 0:
						wSyncCount += (((WORD)rcv_buff - '0') * 10000);
						bSyncIndex++;
						break;
					case 1:
						wSyncCount += (((WORD)rcv_buff - '0') * 1000);
						bSyncIndex++;
						break;
					case 2:
						wSyncCount += (((WORD)rcv_buff - '0') * 100);
						bSyncIndex++;
						break;
					case 3:
						wSyncCount += (((WORD)rcv_buff - '0') * 10);
						bSyncIndex++;
						break;
					case 4:
						wSyncCount += (((WORD)rcv_buff - '0') * 1);
						bSyncIndex++;
						break;
					}
				}
				break;
			case 's':
			case 'S':
				{	//R}hM.
					printf("%luCount\r\n", wSyncCount);

					if(wSyncCount < MAX_SECOND){
						wSecondCount = wSyncCount;
						set_timer0(bMaxPulse);

						get_time();
						bLastMinute = bMinute;	//̎}~.
						bLastSecond = bSecond;

						//I.
						AmpPowerControl(POWER_ON);
						PlayBeep(1);
						AmpPowerControl(POWER_OFF);
					}
					else{
						//ُI.
						AmpPowerControl(POWER_ON);
						PlayBeep(2);
						AmpPowerControl(POWER_OFF);
					}

					bSyncIndex = 0;
					wSyncCount = 0;
				}
				break;
			case 'c':
			case 'C':
				{	//JE^obt@NAR}hM.
					bSyncIndex = 0;
					wSyncCount = 0;
				}
				break;
			case 'p':
			case 'P':
				{	//R}hM.
					get_time();

					AmpPowerControl(POWER_ON);
					PlayBeep(1);
					delay_ms(500);
					PlaySound(hour_tbl[bHour]);
					PlaySound(minute_tbl[bMinute][0]);
					PlaySound(minute_tbl[bMinute][1]);
					PlaySound(LAST_VOICE);
					AmpPowerControl(POWER_OFF);
				}
				break;
			}
		}

		///////////////////////////////////////////////////////
		//  ///////////////////////////////////////////
		get_time();
		if(bLastSecond != bSecond){
			//̃VAo(fobOp).
			#ifdef _SERIAL_DEBUG_
			printf("%02U:%02U:%02U\r\n", bHour, bMinute, bSecond * COUNT_SEC);
			#endif //_SERIAL_DEBUG_

			bLastSecond = bSecond;
		}
		if(bLastMinute != bMinute){
			//030̎.
			if((bMinute == 0) || (bMinute == 30)){
				AmpPowerControl(POWER_ON);
				if(bMinute == 0){
					PlayBeep(2);
				}
				else{
					PlayBeep(1);
				}
				delay_ms(500);
				PlaySound(hour_tbl[bHour]);
				PlaySound(minute_tbl[bMinute][0]);
				PlaySound(minute_tbl[bMinute][1]);
				PlaySound(LAST_VOICE);
				AmpPowerControl(POWER_OFF);
			}
			bLastMinute = bMinute;
		}
	}
}

